#!/usr/bin/perl -w


# Copyright INRA

# Sebastien.Carrere@toulouse.inra.fr
# Jerome.Gouzy@toulouse.inra.fr

# This software is a computer program whose purpose is to provide
# a code generator framework for BioMoby web-services.

# This software is governed by the CeCILL license under French law and
# abiding by the rules of distribution of free software.  You can  use,
# modify and/ or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL
# "http://www.cecill.info".

# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability.

# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or
# data to be ensured and,  more generally, to use and operate it in the
# same conditions as regards security.

# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license and that you accept its terms.


#$Id: DeregisterNamespace.pl 86 2007-10-10 08:05:41Z sletort $

=head1 NAME

DeregisterNamespace.pl - Script to deregister a BioMOBY Namespace

=head1 SYNOPSIS

DeregisterNamespace.pl --central_conf="list_of_central_config_files" --namespace_conf="list_of_namespace_config_files"

=head1 DESCRIPTION

 This script permit to deregister a new BioMOBY namespace
 using 2 types of configuration files:
 
 * central_conf:
    This file contains where to deregister your namespace,
    which mobycentral
    You can give a list of config files to deregister
    many centrals in one shot !
 
 * namespace_conf:
    This file contains information on the namespace,
    like, type, description, URI, relationships
    You can give a list of config files: in that case,
    if a key is duplicated, the last value (in the 
    same order as your file list) overwrite the previous.
    
    

=head1 SUBROUTINES

=cut

my $RH_PARAM = '';

BEGIN
{
    if (exists $ENV{'LIPMLIB'})
    {
        unshift(@INC, $ENV{'LIPMLIB'});
    }

    my ($dirprg, $nameprg) = $0 =~ /(.+)\/(.+)/;
    if (-d "$dirprg/lib")
    {
    	unshift(@INC, "$dirprg/lib");
    }
    if (-d "$dirprg/../lib")
    {
    	unshift(@INC, "$dirprg/../lib");
    }
    if (-d "$dirprg/../lib/Appli")
    {
        unshift(@INC, "$dirprg/../lib/Appli");
    }
}

use strict;
use MOBY::Client::Central;
use Getopt::Long;
use ParamParser;
use BiomobyUtils;

MAIN:
{

    $RH_PARAM = New ParamParser('GETOPTLONG', ("central_conf=s", "namespace_conf=s"));

    &ControlUsage('central_conf');
    &ControlUsage('namespace_conf');

    &executeDeregistering();

}


=head2 procedure executeDeregistering

 Title        : executeDeregistering
 Usage        : &executeDeregistering()
 Prerequisite : none
 Function     : Execute namespace deregistering on each mobycentral given as parameters
 Returns      : undef if OK 
 Args         : none
 Globals      : $RH_PARAM

=cut

sub executeDeregistering
{
    
  
    my $namespace_type        = $RH_PARAM->Get("namespace_type");
   
    my @a_central_conf = split(/\s+/, $RH_PARAM->Get("central_conf"));

    foreach my $central_conf (@a_central_conf)
    {
        $RH_PARAM->SelectNameSpace($central_conf);
        $RH_PARAM->Update($central_conf, 'A');
        my $mobycentral_name = $RH_PARAM->Get("mobycentral_name");
        &SetMobyCentralENV($RH_PARAM,$central_conf);

        my $moby_instance = MOBY::Client::Central->new();
        my ($deregister_call) =
          $moby_instance->deregisterNamespace(
                                              namespaceType   => $namespace_type
                                           );

        if ($deregister_call->success)
        {
            print STDOUT "Deregistering success for \"$mobycentral_name\" central\n";
        }
        else
        {
            print STDOUT "Deregistering failure for \"$mobycentral_name\" central: ", $deregister_call->message, "\n";

        }

        $RH_PARAM->SelectNameSpace('');
    }

    return;
}

=head2 procedure ControlUsage

 Title        : ControlUsage
 Usage        : &ControlUsage()
 Prerequisite : none
 Function     : Control usage of program
 Returns      : undef if OK 
 Args         : $param_name Param to control
 Globals      : $RH_PARAM

=cut

sub ControlUsage
{
    my $param_name = shift();

    if (!(defined($RH_PARAM->Get($param_name))) || ($RH_PARAM->Get($param_name) eq ''))
    {
        &PrintUsage();&Exit("ERROR: $param_name is mandatory");
    }
    else
    {
        my @a_param_values = split(/\s+/, $RH_PARAM->Get($param_name));
        foreach my $param_value (@a_param_values)
        {
            if (!(-e $param_value))
            {
                &PrintUsage();&Exit("ERROR: $param_value does not exist");
            }
            elsif (-z ($param_value))
            {
                &PrintUsage();&Exit("ERROR: $param_value is empty");
            }
            else
            {
                if ($param_name eq 'central_conf')
                {
                    return;
                }
                else
                {
                    $RH_PARAM->Update($param_value, 'O');
                }

            }
        }
    }
    return;
}

=head2 procedure PrintUsage

 Title        : PrintUsage
 Usage        : &PrintUsage()
 Prerequisite : none
 Function     : print usage of the program
 Returns      : none 
 Args         : none
 Globals      : none

=cut

sub PrintUsage
{
    print STDERR <<END;


Usage: 

$0 --central_conf="<CENTRAL_CONFIG_FILE#1> <CENTRAL_CONFIG_FILE#2>\" --namespace_conf=\"<NAMESPACE_CONFIG_FILE#1> <NAMESPACE_CONFIG_FILE#2>\"

	--central_conf: configuration file for mobycentral connection
			* see example at: http://lipm-svn.toulouse.inra.fr/svn/biomoby/trunk/templates/central.cfg
			If a list of file is given as input, duplicate keys are overwritten
			
	--namespace_conf: configuration file for service type specifications
			* see example at: http://lipm-svn.toulouse.inra.fr/svn/biomoby/trunk/templates/namespace.cfg
			If a list of file is given as input, duplicate keys are overwritten
	
	
END
}

