#!/usr/bin/perl -w


# Copyright INRA

# Sebastien.Carrere@toulouse.inra.fr
# Jerome.Gouzy@toulouse.inra.fr

# This software is a computer program whose purpose is to provide
# a code generator framework for BioMoby web-services.

# This software is governed by the CeCILL license under French law and
# abiding by the rules of distribution of free software.  You can  use,
# modify and/ or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL
# "http://www.cecill.info".

# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability.

# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or
# data to be ensured and,  more generally, to use and operate it in the
# same conditions as regards security.

# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license and that you accept its terms.


#$Id: DeregisterServiceType.pl 86 2007-10-10 08:05:41Z sletort $

=head1 NAME

DeregisterServiceType.pl - Script to register a new BioMOBY Service Type

=head1 SYNOPSIS

DeregisterServiceType.pl --central_conf="list_of_central_config_files" --servicetype_conf="list_of_servicetype_config_files"

=head1 DESCRIPTION

 This script permit to deregister a new BioMOBY service type 
 using 2 types of configuration files:
 
 * central_conf:
    This file contains where to deregister your object,
    which mobycentral
    You can give a list of config files to deregister
    many centrals in one shot !
 
 * servicetype_conf:
    This file contains information on the service type,
    like, type, description, URI, relationships
    You can give a list of config files: in that case,
    if a key is duplicated, the last value (in the 
    same order as your file list) overwrite the previous.
    
    

=head1 SUBROUTINES

=cut

my $RH_PARAM = '';

BEGIN
{
    if (exists $ENV{'LIPMLIB'})
    {
        unshift(@INC, $ENV{'LIPMLIB'});
    }

    my ($dirprg, $nameprg) = $0 =~ /(.+)\/(.+)/;
    if (-d "$dirprg/lib")
    {
    	unshift(@INC, "$dirprg/lib");
    }
    if (-d "$dirprg/../lib")
    {
    	unshift(@INC, "$dirprg/../lib");
    }
    if (-d "$dirprg/../lib/Appli")
    {
        unshift(@INC, "$dirprg/../lib/Appli");
    }    
}

use strict;
use MOBY::Client::Central;
use Getopt::Long;
use ParamParser;
use BiomobyUtils;

MAIN:
{

    $RH_PARAM = New ParamParser('GETOPTLONG', ("central_conf=s", "servicetype_conf=s"));

    &ControlUsage('central_conf');
    &ControlUsage('servicetype_conf');

    my $rh_relationships = &GetRelationShips();

    &executeDeregistering($rh_relationships);

}

=head2 function GetRelationShips

 Title        : GetRelationShips
 Usage        : &GetRelationShips()
 Prerequisite : none
 Function     : Parse config file to get Relationships descriptions
 Returns      : $rh_relationships
 Args         : none
 Globals      : $RH_PARAM

=cut

sub GetRelationShips
{
    my @a_relationships_keys = $RH_PARAM->GetKeys('rs\d+_type');

    my %h_relationships = ();

    foreach my $relationships_key (@a_relationships_keys)
    {
        my ($ns) = ($relationships_key =~ /(rs\d+_)type$/);
        $RH_PARAM->SelectNameSpace($ns);

        my $relationship_type = $RH_PARAM->Get("type");

        my @a_related_servicetype_ns      = $RH_PARAM->GetKeys('\d+_srv');
        my @a_list_of_related_servicetype = ();

        foreach my $related_servicetype_ns (@a_related_servicetype_ns)
        {
            my ($servicetype_ns) = ($related_servicetype_ns =~ /(\d+_)srv$/);
            $RH_PARAM->SelectNameSpace('');
            $RH_PARAM->SelectNameSpace("$ns$servicetype_ns");
            my $servicetype_name = $RH_PARAM->Get("srv");

            #my @a_servicetype_def = ($servicetype_name);
            #push(@a_list_of_related_servicetype, \@a_servicetype_def );
            push(@a_list_of_related_servicetype, $servicetype_name);

        }

        if (defined $h_relationships{$relationship_type})
        {
            &PrintUsage();&Exit("ERROR:$relationship_type relationship type defined many times ...");
        }
        else
        {
            $h_relationships{$relationship_type} = \@a_list_of_related_servicetype;
        }
        $RH_PARAM->SelectNameSpace('');
    }

    return (\%h_relationships);
}

=head2 procedure executeDeregistering

 Title        : executeDeregistering
 Usage        : &executeDeregistering($rh_relationships)
 Prerequisite : none
 Function     : Execute service type deregistering on each mobycentral given as parameters
 Returns      : undef if OK 
 Args         : $rh_relationships Hash ref for relationships definitions
 Globals      : $RH_PARAM

=cut

sub executeDeregistering
{
    my $servicetype_name = $RH_PARAM->Get("servicetype_name");

    my @a_central_conf = split(/\s+/, $RH_PARAM->Get("central_conf"));

    foreach my $central_conf (@a_central_conf)
    {
        $RH_PARAM->SelectNameSpace($central_conf);
        $RH_PARAM->Update($central_conf, 'A');
        my $mobycentral_name = $RH_PARAM->Get("mobycentral_name");
        &SetMobyCentralENV($RH_PARAM,$central_conf);

        my $moby_instance = MOBY::Client::Central->new();
        my ($deregister_call) = $moby_instance->deregisterServiceType(serviceType => $servicetype_name,);

        if ($deregister_call->success)
        {
            print STDOUT "Deregistering success for \"$mobycentral_name\" central\n";
        }
        else
        {
            print STDOUT "Deregistering failure for \"$mobycentral_name\" central: ", $deregister_call->message, "\n";

        }

        $RH_PARAM->SelectNameSpace('');
    }

    return;
}

=head2 procedure ControlUsage

 Title        : ControlUsage
 Usage        : &ControlUsage()
 Prerequisite : none
 Function     : Control usage of program
 Returns      : undef if OK 
 Args         : $param_name Param to control
 Globals      : $RH_PARAM

=cut

sub ControlUsage
{
    my $param_name = shift();

    if (!(defined($RH_PARAM->Get($param_name))) || ($RH_PARAM->Get($param_name) eq ''))
    {
        &PrintUsage();&Exit("ERROR: $param_name is mandatory");
    }
    else
    {
        my @a_param_values = split(/\s+/, $RH_PARAM->Get($param_name));
        foreach my $param_value (@a_param_values)
        {
            if (!(-e $param_value))
            {
                &PrintUsage();&Exit("ERROR: $param_value does not exist");
            }
            elsif (-z ($param_value))
            {
                &PrintUsage();&Exit("ERROR: $param_value is empty");
            }
            else
            {
                if ($param_name eq 'central_conf')
                {
                    return;
                }
                else
                {
                    $RH_PARAM->Update($param_value, 'O');
                }

            }
        }
    }
    return;
}

=head2 procedure PrintUsage

 Title        : PrintUsage
 Usage        : &PrintUsage()
 Prerequisite : none
 Function     : print usage of the program
 Returns      : none 
 Args         : none
 Globals      : none

=cut

sub PrintUsage
{
    print STDERR <<END;


Usage: 

$0 --central_conf="<CENTRAL_CONFIG_FILE#1> <CENTRAL_CONFIG_FILE#2>\" --servicetype_conf=\"<SERVICEtype_CONFIG_FILE#1> <SERVICEtype_CONFIG_FILE#2>\"

	--central_conf: configuration file for mobycentral connection
			* see example at: http://lipm-svn.toulouse.inra.fr/svn/biomoby/trunk/templates/central.cfg
			If a list of file is given as input, duplicate keys are overwritten
			
	--servicetype_conf: configuration file for service type specifications
			* see example at: http://lipm-svn.toulouse.inra.fr/svn/biomoby/trunk/templates/servicetype.cfg
			If a list of file is given as input, duplicate keys are overwritten
	
	
END
}

