#!/usr/bin/perl

# Copyright INRA

# Sebastien.Carrere@toulouse.inra.fr
# Jerome.Gouzy@toulouse.inra.fr

# This software is a computer program whose purpose is to provide
# a code generator framework for BioMoby web-services.

# This software is governed by the CeCILL license under French law and
# abiding by the rules of distribution of free software.  You can  use,
# modify and/ or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL
# "http://www.cecill.info".

# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability.

# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or
# data to be ensured and,  more generally, to use and operate it in the
# same conditions as regards security.

# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license and that you accept its terms.

#$Id: DebugWS.pl 86 2007-10-10 08:05:41Z sletort $

=head1 NAME

DebugWS.pl - Script to debug a new BioMOBY Service

=head1 SYNOPSIS

DebugWS.pl --central_conf="<CENTRAL_CONFIG_FILE>" --service_conf="<SERVICE_CONFIG_FILE>" --input_files="<INPUT_XML_FILE#1> <INPUT_XML_FILE#2> ..." [--trace]

=head1 DESCRIPTION

 This script is to test a new BioMOBY service 
 using 2 types of configuration files and a list of input data files:
 
 * central_conf:
    This file contains where your service is registered,
    which mobycentral

 
 * service_conf:
    This file contains information on the service,
    like, name, type, description, inputs, outputs,
    secondaries.
       
 * input_files
    Mandatory input data for the tested service.
    These files must be in BioMoby XML format.
    The name of this xml file will be considered as the articleName

    
=head1 AUTHORS

 Sebastien.Carrere@toulouse.inra.fr 
 Sebastien.Letort@toulouse.inra.fr 


=head1 SUBROUTINES

=cut

my $O_PARAM = '';

BEGIN
{
    if (exists $ENV{'LIPMLIB'})
    {
        unshift(@INC, $ENV{'LIPMLIB'});
    }

    my ($dirprg, $nameprg) = $0 =~ /(.+)\/(.+)/;
    if (-d "$dirprg/lib")
    {
        unshift(@INC, "$dirprg/lib");
    }
    if (-d "$dirprg/../lib")
    {
        unshift(@INC, "$dirprg/../lib");
    }
    if (-d "$dirprg/../lib/Appli")
    {
        unshift(@INC, "$dirprg/../lib/Appli");
    }
}

use strict;
use MOBY::Client::Central;
use MOBY::Client::Service;
use MOBY::MOBYXSLT;
use Getopt::Long;
use ParamParser;
use BiomobyUtils;
use File::Basename;
use XML::Twig;
use Term::ANSIColor;
use PmbTest;
use IO::File;
use Cwd 'abs_path';

MAIN:
{

    $O_PARAM = New ParamParser('GETOPTLONG', ("central_conf=s", "service_conf=s", "input_files=s", "trace", "parameters_file=s"));

    if ($O_PARAM->Get('trace') ne '')
    {
        print STDERR "MODE TRACE\n\n";
        eval("use SOAP::Lite + 'trace'");
    }
    else
    {
        eval("use SOAP::Lite");
    }

    &ControlUsage('central_conf');
    &ControlUsage('service_conf');

    $O_PARAM->Update('A', "service_conf");
    
    if ($O_PARAM->IsDefined('iar1_nm'))
    {
        &ControlUsage('input_files');
    }

    my @a_input_data = ();

    if ($O_PARAM->IsDefined('input_files'))
    {
        my @a_input_files = split(/\s+/, $O_PARAM->Get("input_files"));

        
        foreach my $input_file (@a_input_files)
        {
            my @a_collec = split (/,/,$input_file);

            if ($#a_collec > 0) #COLLECTION
            {
                my @a_collec_data = ();
                my ($article_name,@a_trash) = split ('\.',basename($a_collec[0]));
                push(@a_input_data, $article_name);

                foreach my $collec_input_file (@a_collec)
                {
                    my $data = '';
                    open(INPUT, "$collec_input_file") || die("$! : $collec_input_file\n");
                    while (my $line = <INPUT>)
                    {
                        $data .= $line;
                    }
                    close INPUT;
                    push (@a_collec_data,$data);
                }
                push(@a_input_data, \@a_collec_data);

            }
            else #PAS COLLECTION
            {
                my $input_file_name = basename( $input_file);
                my ($article_name,@a_trash) = split ('\.',$input_file_name);
    	        push(@a_input_data, $article_name);
                my $data = '';
                open(INPUT, "$input_file") || die("$! : $input_file\n");
                while (my $line = <INPUT>)
                {
                    $data .= $line;
                }
                close INPUT;
                push(@a_input_data, $data);

            }
        } 
    }
    if ($O_PARAM->IsDefined('parameters_file') && $O_PARAM->AssertFileExists('parameters_file'))
    {
        open (PARAM, $O_PARAM->Get('parameters_file')) || die ("$! : " . $O_PARAM->Get('parameters_file') ."\n");
        while (my $line = <PARAM>)
        {
            chomp $line;
            my ($param_name,$value) = ($line =~ /(\w+)=(.+)/);
            push (@a_input_data, $param_name);
            push (@a_input_data, "<Value>$value</Value>");
        }
        close PARAM;
    }

    &SetMobyCentralENV($O_PARAM, $O_PARAM->Get("central_conf"));

    print STDERR "REGISTRY:\n\tMOBY_SERVER " . $ENV{'MOBY_SERVER'} . "\n\tMOBY_URI " . $ENV{'MOBY_URI'} . "\n" if ($O_PARAM->IsDefined('trace'));
    my $auth_uri     = $O_PARAM->Get('auth_uri');
    my $service_name = $O_PARAM->Get('service_name');
    my $service_category = $O_PARAM->Get('service_category');

    print STDERR "DEBUG DU SERVICE : $service_name by $auth_uri\n" if ($O_PARAM->IsDefined('trace'));

    my $moby_instance = MOBY::Client::Central->new();
    
    my ($ra_service_instance, $r) = $moby_instance->findService(authURI => $auth_uri, serviceName => $service_name, category => $service_category );

    my $service_instance = shift @{$ra_service_instance};

    print STDERR "SERVICE INSTANCE\n" if ($O_PARAM->IsDefined('trace'));

    my $wsdl = $moby_instance->retrieveService($service_instance);

    print STDERR "WSDL $wsdl\n" if ($O_PARAM->IsDefined('trace'));
    
    my $service_invocation;
    if ($service_category eq 'moby-async')
    {
	eval 'use MOBY::Async::Service';
	die "ERROR: $@\n" if ($@);
	$service_invocation = MOBY::Async::Service->new(service => $wsdl);
    }
    else
    {
	$service_invocation = MOBY::Client::Service->new(service => $wsdl);
    }

    die "ERROR: SERVICE INSTANCE CREATION FAILED\n" if (! defined $service_invocation);

    print STDERR "SERVICE INVOCATION\n" if ($O_PARAM->IsDefined('trace'));


    my $start_time = time;
    my $result = $service_invocation->execute(XMLinputlist => [\@a_input_data]);
    my $end_time = time;
    
    my $dirname          = abs_path(dirname($O_PARAM->Get("service_conf")));
    my $test_dir         = "$dirname/test/";
    if (-d $test_dir)
    {
        my $test_result_file = $test_dir . $service_name . ".xml";
        my $fh_test_result_file = new IO::File(">$test_result_file");
        if (defined $fh_test_result_file)
        {
            print $fh_test_result_file $result;
            $fh_test_result_file->close;
            
            print STDOUT color ('bold yellow'), "\nBioMoby XML result is written in this file:\n$test_result_file\nThis file will be used as the reference for service monitoring\n";
            print STDOUT color('reset');
        }
    }
    
    my $duration = $end_time - $start_time + 1;
    my ($exceptioncode,$exceptionmessage,$rh_articles_data) = &AnalyseResult($result,$duration);

    my $exceptioncolor = 'red';
    $exceptioncolor = 'green' if ($exceptioncode == 700);

    print STDOUT color($exceptioncolor);
    print STDOUT "\n\n\nException $exceptioncode : $exceptionmessage\n";
    print STDOUT color('reset');
    print STDOUT "\n\n\n";
    
    foreach my $article_name (keys %{$rh_articles_data})
    {
        my $sep = $article_name;
        $sep =~ s/\S/-/g;

        print STDOUT colored ['bold white on_blue'], "\n$sep\n$article_name" . "\n$sep";
        print STDOUT "\n";

        print STDOUT color('bold blue');
        print STDOUT $rh_articles_data->{$article_name} . "\n";
        print STDOUT color('reset');
        print STDOUT "\n";
    }





}

=head2 procedure ControlUsage

 Title        : ControlUsage
 Usage        : &ControlUsage()
 Prerequisite : none
 Function     : Control usage of program
 Returns      : undef if OK 
 Args         : $param_name Param to control
 Globals      : $O_PARAM

=cut

sub ControlUsage
{
    my $param_name = shift();

    if (!(defined($O_PARAM->Get($param_name))) || ($O_PARAM->Get($param_name) eq ''))
    {
        &PrintUsage();&Exit("ERROR: $param_name is mandatory");
    }
    else
    {
        my @a_param_values = split(/[,\s+]/, $O_PARAM->Get($param_name));
        foreach my $param_value (@a_param_values)
        {
            if (!(-e $param_value))
            {
                &PrintUsage();&Exit("ERROR: $param_value does not exist");
            }
            elsif (-z ($param_value))
            {
                &PrintUsage();&Exit("ERROR: $param_value is empty");
            }
            else
            {
                if ($param_name eq 'central_conf')
                {
                    return;
                }
                else
                {
                    $O_PARAM->Update($param_value, 'O');
                }

            }
        }
    }
    return;
}


              
sub AnalyseResult
{
	my ($result_string,$duration) = (@_);
 
	my ($service_notes,$ra_queries,$ra_exceptions) = &MOBYXSLT::getInputs($result_string);

	my $rh_exception = shift(@{$ra_exceptions});
	my $exceptioncode = $rh_exception->{code};
	my $exceptionmessage = $rh_exception->{message};
	my $data = '';

# 
#     my $rh_pmbtests = {};
#     
#     
#     if ($O_PARAM->IsDefined('pmbtest_file') && $O_PARAM->AssertFileExists('pmbtest_file'))
#     {
# 		$rh_pmbtests = &PmbTest::LoadPmbTestXml($O_PARAM->Get('pmbtest_file'));
# 	}
#     
#     $rh_pmbtests->{service_name} = $O_PARAM->Get('service_name');
#     $rh_pmbtests->{auth_uri} = $O_PARAM->Get('auth_uri');
#     $rh_pmbtests->{registry} = $ENV{MOBY_SERVER};
#     $rh_pmbtests->{duration} = $duration;

	my $query = shift (@{$ra_queries});
    my $rh_articles_data = &PmbTest::GetArticlesData($query);
    
	return ($exceptioncode,$exceptionmessage,$rh_articles_data);

}

=head2 procedure PrintUsage

 Title        : PrintUsage
 Usage        : &PrintUsage()
 Prerequisite : none
 Function     : print usage of the program
 Returns      : none 
 Args         : none
 Globals      : none

=cut

sub PrintUsage
{
    print STDERR <<END;


Usage: 

$0 --central_conf="<CENTRAL_CONFIG_FILE>" --service_conf="<SERVICE_CONFIG_FILE>" --input_files="<INPUT_XML_FILE#1> <INPUT_XML_FILE#2> ..." [--trace]

	--central_conf: configuration file for mobycentral connection
			* see example at: http://lipm-svn.toulouse.inra.fr/svn/biomoby/trunk/templates/central.conf
						
	--service_conf: configuration file for service specifications
			* see example at: http://lipm-svn.toulouse.inra.fr/svn/biomoby/trunk/templates/service.conf
				
	--input_files:  Mandatory input data for the tested service.
	    	    	These files must be in BioMoby XML format.
			* see example at: 
			    http://lipm-svn.toulouse.inra.fr/svn/biomoby/trunk/templates/fasta.xml
			    http://lipm-svn.toulouse.inra.fr/svn/biomoby/trunk/templates/gff.xml
	--parameters_file:  Optional file containing parameters for service execution
                        (parameters = secondary articles)
                        Format: parameter=value

    --trace: verbose mode , use of "SOAP::Lite + 'trace'" instead of "SOAP::Lite"
			
	
END
}
