#!/usr/bin/perl -w

# Copyright INRA

# Sebastien.Carrere@toulouse.inra.fr
# Jerome.Gouzy@toulouse.inra.fr

# This software is a computer program whose purpose is to provide
# a code generator framework for BioMoby web-services.

# This software is governed by the CeCILL license under French law and
# abiding by the rules of distribution of free software.  You can  use,
# modify and/ or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL
# "http://www.cecill.info".

# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability.

# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or
# data to be ensured and,  more generally, to use and operate it in the
# same conditions as regards security.

# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license and that you accept its terms.

# $Id: DeregisterService.pl 86 2007-10-10 08:05:41Z sletort $

=head1 NAME

pmb_DeregisterService.pl - Script to deregister an existing BioMOBY Service

=head1 SYNOPSIS

pmb_DeregisterService.pl --central_conf="list_of_central_config_files" --service_conf="list_of_service_config_files"

=head1 DESCRIPTION

 This script permit to deregister an existing BioMOBY service 
 using 2 types of configuration files:
 
 * central_conf:
    This file contains where to deregister your service,
    which mobycentral
    You can give a list of config files to register
    many centrals in one shot !
 
 * service_conf:
    This file contains mandatory information on the service,
    like, name and authURI
    You can give a list of config files: in that case,
    if a key is duplicated, the last value (in the 
    same order as your file list) overwrite the previous.


=head1 AUTHORS

 Sebastien.Carrere@toulouse.inra.fr 
 Sebastien.Letort@toulouse.inra.fr

 
=head1 SUBROUTINES

=cut

my $O_PARAM = '';

BEGIN
{
    if (exists $ENV{'LIPMLIB'})
    {
        unshift(@INC, $ENV{'LIPMLIB'});
    }

    my ($dirprg, $nameprg) = $0 =~ /(.+)\/(.+)/;
    if (-d "$dirprg/lib")
    {
        unshift(@INC, "$dirprg/lib");
    }
    if (-d "$dirprg/../lib")
    {
        unshift(@INC, "$dirprg/../lib");
    }
    if (-d "$dirprg/../lib/Appli")
    {
        unshift(@INC, "$dirprg/../lib/Appli");
    }
}

use strict;
use MOBY::Client::Central;
use Getopt::Long;
use ParamParser;
use BiomobyUtils;
use Cwd;

MAIN:
{
    $O_PARAM =
      New ParamParser('GETOPTLONG',
                      ("central_conf=s", "service_conf=s", "module=s", "remove_from_dispatcher", "special_conf=s"));

    &ControlUsage('central_conf');
    &ControlUsage('service_conf');
    &ControlUsage('module');
    $ENV{PATH_WS_BIOMOBY} = $O_PARAM->Get('path_ws_biomoby') if ($O_PARAM->IsDefined('path_ws_biomoby'));
    $ENV{URL_WS_BIOMOBY} = $O_PARAM->Get('url_ws_biomoby') if ($O_PARAM->IsDefined('url_ws_biomoby'));
    
    
    if ($O_PARAM->IsDefined('special_conf'))
    {
        $O_PARAM->AssertFileExists('special_conf');
        $O_PARAM->Update($O_PARAM->Get('special_conf'), 'O');

        if ($O_PARAM->IsDefined('path_ws_biomoby'))
        {
            $ENV{PATH_WS_BIOMOBY} = $O_PARAM->Get('path_ws_biomoby');
        }
        if ($O_PARAM->IsDefined('url_ws_biomoby'))
        {
            $ENV{URL_WS_BIOMOBY} = $O_PARAM->Get('url_ws_biomoby');
        }
    }

    if ((!defined $ENV{PATH_WS_BIOMOBY}) || (!defined $ENV{URL_WS_BIOMOBY}))
    {
        &Exit(
            "ERROR: Environnement variable PATH_WS_BIOMOBY and URL_WS_BIOMOBY have to be set\nYou can use a special.cfg file to define thes values or your shell\n"
            );
    }

    my $module_name_long = $O_PARAM->Get("module");
    (my $module_name) = ($module_name_long =~ /(\w+)\.pm/);

    #Sebastien 20/07/05
    #Je rajoute le repertoire parent du module dans le fichier du dispatcher pour
    #eviter d'avoir un module qui fait les eval et avoir tous les .pm
    #dans un repertoire lib

    my $module_directory = $module_name_long;
    $module_directory =~ s/$module_name\.pm$//;

    if ($module_directory eq '')
    {
        $module_directory = './';
    }

    my $module_abs_path = Cwd::abs_path($module_directory);

    my $services_dir = $ENV{PATH_WS_BIOMOBY} . '/services';
    $module_abs_path =~ s/^$services_dir//;
 
    $module_abs_path = '/' if ($module_abs_path eq '');
    
    my @a_path_to_module = split(/\//, $module_abs_path);
    
    push (@a_path_to_module, '') if ( $#a_path_to_module <0 );
    $module_directory = $a_path_to_module[$#a_path_to_module] . "/";
    
    $module_directory = '' if (! defined $module_directory);

 
    my $ra_registry_names = &executeDeregistering($module_directory);

    if ($O_PARAM->IsDefined("remove_from_dispatcher"))
    {
        my $service_name   = $O_PARAM->Get("service_name");
        my $dispatcher_url = $O_PARAM->Get("dispatcher_url");
    	foreach my $registry_name (@{$ra_registry_names})
    	{
	        &RemoveFromDispatcher($dispatcher_url, $service_name,$registry_name);
    	}
    }

}

=head2 procedure executeDeregistering

 Title        : executeDeregistering
 Usage        : &executeDeregistering($ra_input_articles,$ra_output_articles,$rh_secondary)
 Prerequisite : none
 Function     : Execute service deregistering on each mobycentral given as parameters
 Returns      : undef if OK 
 Args         : none
 Globals      : $O_PARAM

=cut

sub executeDeregistering
{
    my $module_directory = shift();
    my $service_name     = $O_PARAM->Get('service_name');

    #my $auth_uri     = $O_PARAM->Get('auth_uri');
    my @a_central_conf = split(/\s+/, $O_PARAM->Get('central_conf'));
    my @a_registry_names = ();
    foreach my $central_conf (@a_central_conf)
    {

        #$O_PARAM->SelectNameSpace($central_conf);
        $O_PARAM->Update($central_conf, 'A');

        my $mobycentral_name = $O_PARAM->Get('mobycentral_name');

        &SetMobyCentralENV($O_PARAM, $central_conf);

        my $signature_path = $O_PARAM->Get("signature_path") . '/';
        my $signature_url  = $O_PARAM->Get("signature_url");
        $signature_url =~ s/\/$//g;
        $signature_url .= '/';        
        $module_directory = '' if ($module_directory eq '/');
        my $dispatcher_url = $O_PARAM->Get("dispatcher_url");

        #my $moby_instance = MOBY::Client::Central->new(Registries => %h_registries);
        my $rdf_file = $signature_path . $module_directory . $mobycentral_name . '_' . $service_name . ".rdf";
	my $signature_url_complete = $signature_url . $module_directory . $mobycentral_name . '_' .$service_name . ".rdf";
	
	if (!-e $rdf_file)
        {
        	$rdf_file = $signature_path . $module_directory  . $service_name . ".rdf";
		$signature_url_complete = $signature_url . $module_directory . $service_name . ".rdf";
	}
	
	my $bkp_file = $rdf_file . ".bkp";

        if (!-e $rdf_file)
        {
            print STDOUT "\nRDF File $rdf_file does not exist\n";
            print STDOUT "\nDeregistering Failed for \"$mobycentral_name\" central\n";
            exit;
        }
        else
        {
            system("mv $rdf_file $bkp_file");
            my $bkp_content = `cat $bkp_file`;
            my $rdf_content = $bkp_content;
            $rdf_content =~ s/\n/#nl#/g;
            $rdf_content =~ s/<rdf:Description.+\/rdf:Description>//g;
	    # pour garder la retrocompatibilite
            $rdf_content =~ s/<(mygrid:serviceDescription).+\/mygrid:serviceDescription>//g;
            $rdf_content =~ s/#nl#/\n/g;
	    
            open(RDF, ">$rdf_file");
            print RDF $rdf_content;
            close RDF;

            print STDOUT "\nRDF File moved to $bkp_file\n";

        }

        my $moby_instance = MOBY::Client::Central->new();

        #my $register_call = $moby_instance->deregisterService(serviceName => $service_name,authURI     => $auth_uri);

	
        my $register_call =
          $moby_instance->registerService(signatureURL => $signature_url_complete);

        if ($register_call->success)
        {
            print STDOUT "\nDeregistering success for \"$mobycentral_name\" central\n";
            push (@a_registry_names,$mobycentral_name);
        }
        else
        {
            print STDOUT "\nDeregistering failure for \"$mobycentral_name\" central: ", $register_call->message, "\n";
            print STDOUT "\nYou can restore your RDF file using $bkp_file\n";

            foreach my $key (keys(%{$register_call}))
            {
                if (defined $register_call->{$key})
                {
                    print STDOUT "\t$key: $register_call->{$key}\n";
                }
            }
        }

        $O_PARAM->SelectNameSpace('');
    }

    return \@a_registry_names;
}

=head2 procedure ControlUsage

 Title        : ControlUsage
 Usage        : &ControlUsage()
 Prerequisite : none
 Function     : Control usage of program
 Returns      : undef if OK 
 Args         : $param_name Param to control
 Globals      : $O_PARAM

=cut

sub ControlUsage
{
    my $param_name = shift();

    if (!(defined($O_PARAM->Get($param_name))) || ($O_PARAM->Get($param_name) eq ''))
    {
        &PrintUsage();
        &Exit("ERROR: $param_name is mandatory");
    }
    else
    {
        my @a_param_values = split(/\s+/, $O_PARAM->Get($param_name));
        foreach my $param_value (@a_param_values)
        {
            if (!(-e $param_value))
            {
                &PrintUsage();
                &Exit("ERROR: $param_value does not exist");
            }
            elsif (-z ($param_value))
            {
                &PrintUsage();
                &Exit("ERROR: $param_value is empty");
            }
            else
            {
                if ($param_name eq 'central_conf')
                {
                    return;
                }
                else
                {
                    $O_PARAM->Update($param_value, 'O');
                }
            }
        }
    }
    return;
}

=head2 procedure PrintUsage

 Title        : PrintUsage
 Usage        : &PrintUsage()
 Prerequisite : none
 Function     : print usage of the program
 Returns      : none 
 Args         : none
 Globals      : none

=cut

sub PrintUsage
{
    print STDERR <<END;


Usage: 

$0 --central_conf="<CENTRAL_CONFIG_FILE#1> <CENTRAL_CONFIG_FILE#2>\" --service_conf=\"<SERVICE_CONFIG_FILE#1> <SERVICE_CONFIG_FILE#2> [--remove_from_dispatcher --special_conf=<specific_values>]\"

	--central_conf: configuration file for mobycentral connection
			* see example at: http://lipm-svn.toulouse.inra.fr/svn/biomoby/trunk/templates/central.conf
			If a list of file is given as input, duplicate keys are overwritten
			
	--service_conf: configuration file for service specifications
			* see example at: http://lipm-svn.toulouse.inra.fr/svn/biomoby/trunk/templates/service.conf
			If a list of file is given as input, duplicate keys are overwritten
    
    --module:  The Perl module containing your Service Sub

Optional:
	
	--remove_from_dispatcher: trigger to remove your service name from the dispatcher CGI.
    
    --special_conf: configuration file for current values (RDF path file, service_contact, dispatcher url



END
}
