#!/usr/bin/perl

# Copyright INRA

# Sebastien.Carrere@toulouse.inra.fr
# Jerome.Gouzy@toulouse.inra.fr

# This software is a computer program whose purpose is to provide
# a code generator framework for BioMoby web-services.

# This software is governed by the CeCILL license under French law and
# abiding by the rules of distribution of free software.  You can  use,
# modify and/ or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL
# "http://www.cecill.info".

# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability.

# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or
# data to be ensured and,  more generally, to use and operate it in the
# same conditions as regards security.

# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license and that you accept its terms.

# $Id: DeregisterService.pl 86 2007-10-10 08:05:41Z sletort $

=head1 NAME

pmb_MobyleParser.pl - Mobyle to Webservice tool

=head1 SYNOPSIS

pmb_MobyleParser.pl --xmlfile=path_to_xml_mobyle_file [options]

=head1 DESCRIPTION

Mobyle parsing and creation of a webservice tarball

This script will produce a standalone tarball: we analyse the Mobyle-xml to generate a Webservice configuration file.
N.B: We use such a configuration file to be independant of Mobyle DTD modification: if the Mobyle file change, we just have to modify our Mobyle parser and not all the downstream programs (using our configuration file).

This step leads to the creation of tarball containing the configuration file but also data to test the webservice and a log file.
To produce this tarball, please use the "pmb_MobyleParser.pl" script:

% tcsh
% source .setenv
% $PMBBIN/pmb_MobyleParser.pl --xmlfile=path_to_xml_mobyle_file

The following parameters are optional (but if you do not defined them, the system will ask for)

                --tmp_dir=<temporary directory>
                        this directory should be apache writable (/tmp is okay)

                --service_contact=<werbservice support email>

                --bin_fullpath=<full path to the binary/script embedded in the webservice> 
                        this parameter is already defined in the Mobyle-xml file, but we redefine it here because you could imagine
                        to generate the Mobyle file on a serverA and then deploy your webservice on serverB; in this case,
                        there is a chance that the executable is not in the same path.

Then the program will ask you for test data for each input. You just have to give raw data an playmoby will convert it into BioMoby XML format.


=head1 AUTHORS

 Sebastien.Carrere@toulouse.inra.fr 
 Sebastien.Letort@toulouse.inra.fr

 
=head1 SUBROUTINES

=cut


use strict;
use warnings;
use File::Basename;
use Cwd;
use Term::ANSIColor;


my ($dirprg);
BEGIN
{
    if (exists $ENV{'LIPMLIB'})
    {
        unshift(@INC, $ENV{'LIPMLIB'});
    }
	
    $dirprg = dirname ($0);
    

    if (-d "$dirprg/../lib")
    {
        unshift(@INC, "$dirprg/../lib");
    }
    if (-d "$dirprg/../lib/Appli")
    {
        unshift(@INC, "$dirprg/../lib/Appli");
    }
    $dirprg = '.' if (!defined $dirprg);

}

use Data::Dumper;
use File::Copy;

use Error qw( :try );
use IO::Dir;
use MobyleParser;
use ParamParser;

MAIN:
{
	# 0- recuperer les options
	my $o_pp = New ParamParser( 'GETOPTLONG', qw(xmlfile=s tmp_dir=s bin_fullpath=s service_contact=s async sync) );
	$o_pp->SetUsage( \&Usage );
	$o_pp->AssertDefined( qw( xmlfile ) );
	$o_pp->SetUnlessDefined('tmp_dir', '/tmp') if (-d '/tmp');
	

	try
	{
		# 1- interpreter le fichier mobyle xml
		my $o_mp = New MobyleParser( $o_pp->Get('xmlfile') );

        my $service_name = $o_mp->{ service_name };
        mkdir ($$) or die "$! mkdir( $$/$service_name/test ) ";
		mkdir( "$$/$service_name" ) or die "$! mkdir( $$/$service_name ) ";
    	mkdir( "$$/$service_name" . '/test' ) or die "$! mkdir( $$/$service_name/test ) ";
        
		# 3- si pas l'option --biomoby on demande de quoi a l'utilisateur
		$o_mp = &GetInfosFromUser($o_pp,$o_mp);


		$o_mp->GetTestInfos("$$/$service_name/test");

		# 4- on a toutes les infos, on ecrit tous dans un fichier cfg
		$o_mp->Print("$$/$service_name");

        chdir $$;
		`tar cvfj $service_name.bz2 $service_name`;
        chdir '..';
        `mv $$/$service_name.bz2 $service_name.bz2`;
		`rm -Rf $$`;
		print STDOUT color 'bold green';
		print STDOUT "\n\n$service_name.bz2 has been created.\nNext step: deploy your $service_name WS using playmoby environnement.\n\n";
		print STDOUT color 'reset';
	}
	catch Error::LipmError with
	{
		my ( $E, $ref ) = @_;

		print color 'bold red';
		print $E->Explain();
		print color 'reset';
	}
	otherwise
	{
		print Dumper \@_;
	};


}	#MAIN

=head2 function GetInfosFromUser

 Title        : GetInfosFromUser

=cut

sub GetInfosFromUser
{
	my ($o_pp, $o_mp) = (@_);

	if ( $o_pp->IsDefined('tmp_dir'))
	{
		$o_mp->{ tmp_dir } = $o_pp->Get('tmp_dir');
	}
	else
	{
		my $question = "What is your tmp directory ?";
		my $tmp_dir = $o_mp->{ tmp_dir };
		$question .= " [$tmp_dir]"	if defined $tmp_dir;
		$question .= " : ";
		print $question;
		my $reponse = <STDIN>;	chomp $reponse;
		$o_mp->{ tmp_dir } = ($reponse ne '') ? $reponse : $tmp_dir;
	}
	
	if ( $o_pp->IsDefined('service_contact'))
	{
		$o_mp->{ service_contact } = $o_pp->Get('service_contact');
	}
	else
	{
		# contact
		my $question = "what is the email of the responsable ?";
		my $contact = $o_mp->{ service_contact };
		$question .= " [$contact]"	if defined $contact;
		$question .= " : ";
		print $question;
		my $reponse = <STDIN>;	chomp $reponse;
		if( $reponse ne '' )
		{	$o_mp->{ service_contact } = $reponse;	}
		elsif( defined $contact )
		{	$o_mp->{ service_contact } = $contact;	}
	}
	# le chemin de l'executable
	if ( $o_pp->IsDefined('bin_fullpath'))
	{
		$o_mp->{ bin_path } = $o_pp->Get('bin_fullpath');
	}
	else
	{
		my $question = "Please indicate the absolute path to the binary file";
		$question .= " or nothing if its in the PATH of the WS server :";
		print $question;
		my $reponse = <STDIN>;	chomp $reponse;
		$o_mp->{ bin_path } = $reponse;
	}

	# async/sync
	
	if ( $o_pp->IsDefined('async'))
	{
		$o_mp->{ async } = 1;
	}
	elsif  ( $o_pp->IsDefined('sync'))
	{
		$o_mp->{ async } = 0;
	}
	else
	{
		my $question = "Do you want to generate an asynchronous web-service ? [y/N]";
		print $question;
		my $reponse = <STDIN>;	chomp $reponse;
		if ($reponse =~ /y/i)
		{
			$o_mp->{ async } = 1;
		}
	}

        if ($o_mp->{ async } == 1)
	{
		eval "use WSRF::Lite;";
        	if ($@)
        	{
            		print STDERR color 'red';
            		print STDERR "\nERROR CONFIGURATION: WSRF::Lite can not be found on your system: $@\nThis module is mandatory to deploy asynchronous webservices\nYou can find it at CPAN URL: http://search.cpan.org/search?mode=module&query=WSRF::Lite\n\n";
	    		print STDERR color 'reset';
			exit;
		}
	}

	return $o_mp;
}



sub Usage
{
	print STDOUT <<END;

Usage :
	perl $0 --xmlfile=path_to_xml_mobyle_file [options]

	
	--tmp_dir=<temporary directory: apache writable>
	--service_contact=<WS support email>
	--bin_fullpath=<path to the binary/script embedded in the WS>
	               WARNING: this path should end with the name of the binary
	--async your web-service will be run in asynchronous mode
	--sync your web-service will be run in synchronous mode
	
END

}


__END__

