#!/usr/bin/perl

# Copyright INRA

# Sebastien.Carrere@toulouse.inra.fr
# Jerome.Gouzy@toulouse.inra.fr

# This software is a computer program whose purpose is to provide
# a code generator framework for BioMoby web-services.

# This software is governed by the CeCILL license under French law and
# abiding by the rules of distribution of free software.  You can  use,
# modify and/ or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL
# "http://www.cecill.info".

# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability.

# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or
# data to be ensured and,  more generally, to use and operate it in the
# same conditions as regards security.

# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license and that you accept its terms.

# $Id: DeregisterService.pl 86 2007-10-10 08:05:41Z sletort $

=head1 NAME

pmb_MobyleParser.pl - Mobyle to Webservice tool

=head1 SYNOPSIS

pmb_MobyleParser.pl --xmlfile=path_to_xml_mobyle_file [options]

=head1 DESCRIPTION

Mobyle parsing and creation of a webservice tarball

This script will produce a standalone tarball: we analyse the Mobyle-xml to generate a Webservice configuration file.
N.B: We use such a configuration file to be independant of Mobyle DTD modification: if the Mobyle file change, we just have to modify our Mobyle parser and not all the downstream programs (using our configuration file).

This step leads to the creation of tarball containing the configuration file but also data to test the webservice and a log file.
To produce this tarball, please use the "pmb_MobyleParser.pl" script:

% tcsh
% source .setenv
% $PMBBIN/pmb_MobyleParser.pl --xmlfile=path_to_xml_mobyle_file

The following parameters are optional (but if you do not defined them, the system will ask for)

                --tmp_dir=<temporary directory>
                        this directory should be apache writable (/tmp is okay)

                --service_contact=<werbservice support email>

                --bin_fullpath=<full path to the binary/script embedded in the webservice> 
                        this parameter is already defined in the Mobyle-xml file, but we redefine it here because you could imagine
                        to generate the Mobyle file on a serverA and then deploy your webservice on serverB; in this case,
                        there is a chance that the executable is not in the same path.

Then the program will ask you for test data for each input. You just have to give raw data an playmoby will convert it into BioMoby XML format.


=head1 AUTHORS

 Sebastien.Carrere@toulouse.inra.fr 
 Sebastien.Letort@toulouse.inra.fr

 
=head1 SUBROUTINES

=cut

use strict;
use warnings;
use File::Basename;
use Cwd;
use Term::ANSIColor;

my ($dirprg);

BEGIN
{
    if (exists $ENV{'LIPMLIB'})
    {
        unshift(@INC, $ENV{'LIPMLIB'});
    }

    $dirprg = dirname($0);

    if (-d "$dirprg/../lib")
    {
        unshift(@INC, "$dirprg/../lib");
    }
    if (-d "$dirprg/../lib/Appli")
    {
        unshift(@INC, "$dirprg/../lib/Appli");
    }
    $dirprg = '.' if (!defined $dirprg);

}

use Data::Dumper;
use File::Copy;

use Error qw( :try );
use IO::Dir;
use MobyleParser;
use ParamParser;
use Runner;

MAIN:
{

    # 0- recuperer les options
    my $o_param = New ParamParser(
          'GETOPTLONG',
          qw(xmlfile=s outdir=s parameters=s verbose print_input_names print_output_names print_parameter_names nocolor)
          );
    $o_param->SetUsage(\&Usage);
    $o_param->AssertDefined(qw( xmlfile parameters))
      if (   !$o_param->IsDefined('print_input_names')
          && !$o_param->IsDefined('print_output_names')
          && !$o_param->IsDefined('print_parameter_names'));
    print STDERR color 'bold blue'                                             if (!$o_param->IsDefined('nocolor'));
    print STDERR "Load Parameters from\t" . $o_param->Get('parameters') . "\n" if ($o_param->IsDefined('verbose'));
    print STDERR color 'reset'                                                 if (!$o_param->IsDefined('nocolor'));
    $o_param->Update($o_param->Get('parameters'), 'A');
    $o_param->SetUnlessDefined('outdir', '/tmp') if (-d '/tmp');

    try
    {

        # 1- interpreter le fichier mobyle xml
        print STDERR color 'bold blue' if (!$o_param->IsDefined('nocolor'));
        print STDERR "Load Mobyle file\t" . $o_param->Get('xmlfile') . "\n" if ($o_param->IsDefined('verbose'));
        my $o_mp = New MobyleParser($o_param->Get('xmlfile'));

        if ($o_param->IsDefined('print_input_names'))
        {
            print STDOUT color 'reset' if (!$o_param->IsDefined('nocolor'));
            foreach my $input_name (keys %{$o_mp->{primary_article}})
            {
                my $type = $o_mp->{primary_article}->{$input_name}->{objects}->[0]->{otp};
                print "$input_name:$type\n";
            }
            exit 0;
        }

        if ($o_param->IsDefined('print_output_names'))
        {
            print STDOUT color 'reset' if (!$o_param->IsDefined('nocolor'));

            foreach my $output_name (keys %{$o_mp->{output}})
            {
                my $type = $o_mp->{output}->{$output_name}->{objects}->[0]->{otp};
                print "$output_name:$type\n";
            }
            exit 0;
        }

        if ($o_param->IsDefined('print_parameter_names'))
        {
            print STDOUT color 'reset' if (!$o_param->IsDefined('nocolor'));
            foreach my $parameter_name (keys %{$o_mp->{secondary_article}})
            {
                my $type = $o_mp->{$parameter_name}->{secondary_article}->{type};
                print "$parameter_name:#$type#\n" if (($parameter_name !~ /_name$/) && ($type !~ /NULL/));
            }
            exit 0;
        }

        my $cmd = $o_mp->GetSystemCmd($o_param);

        #
        #suppression des parametres sans valeur
        #
        while ($cmd =~ /(\-+\w+=\s+)/g or $cmd =~ /(\-+\w+=)$/g or $cmd =~ /(\-+\w+='')$/g or $cmd =~ /(\-+\w+="")$/g)
        {
            $cmd =~ s/$1//;
        }

        print STDERR color 'bold blue' if (!$o_param->IsDefined('nocolor'));

        print STDERR "Prepare command\t" . $cmd . "\n"                     if ($o_param->IsDefined('verbose'));
        print STDERR "Create directory\t" . $o_param->Get('outdir') . "\n" if ($o_param->IsDefined('verbose'));
        mkdir($o_param->Get('outdir'))                                     if (!-d $o_param->Get('outdir'));
        print STDERR "Change directory\t" . $o_param->Get('outdir') . "\n" if ($o_param->IsDefined('verbose'));
        chdir $o_param->Get('outdir');
        print STDERR "Launch command\t" . $cmd . "\n" if ($o_param->IsDefined('verbose'));

        #system ($cmd);
        my ($status, $killed) = Runner::RunExt(-cmd => $cmd, -snippet => 1);
        print STDERR "Process finished\n" if ($o_param->IsDefined('verbose'));
        print STDERR color 'reset' if (!$o_param->IsDefined('nocolor'));
        exit $status;
    }
    catch Error::LipmError with
    {
        my ($E, $ref) = @_;

        print color 'bold red' if (!$o_param->IsDefined('nocolor'));
        print $E->Explain();
        print color 'reset' if (!$o_param->IsDefined('nocolor'));
    }
    otherwise
    {
        print Dumper \@_;
    };

}    #MAIN

sub Usage
{
    print STDOUT <<END;

Usage :
	perl $0 --xmlfile=<path_to_xml_mobyle_file> --parameters=<configuration file> [--outdir=<output directory>]

Mandatory:
    
    --xmlfile               : path to mobyle program xml file
    --parameters            : input paramter file
        
        should contain 
            - [mandatory] input data path file (MobyleInputParameterName=/path/to/input/file/)
            - [optionnal] optionnal parameter values (MobyleParameterName=value) to overload vdef
            - [optionnal] output data path file (MobyleOutputParameterName=/path/to/input/file/) to control output file name

Optionnal:
    --outdir                : path to output directory [default: /tmp]
    --print_input_names     : print input names and BioMOBY type
    --print_output_names    : print output names and BioMOBY type
    --print_parameter_names : print parameter names and BioMOBY type

    --verbose
    --nocolor               : disable coloured display
    
END

}

__END__

