#!/usr/bin/perl


# Copyright INRA

# Sebastien.Carrere@toulouse.inra.fr
# Jerome.Gouzy@toulouse.inra.fr

# This software is a computer program whose purpose is to provide
# a code generator framework for BioMoby web-services.

# This software is governed by the CeCILL license under French law and
# abiding by the rules of distribution of free software.  You can  use,
# modify and/ or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL
# "http://www.cecill.info".

# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability.

# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or
# data to be ensured and,  more generally, to use and operate it in the
# same conditions as regards security.

# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license and that you accept its terms.

=head1 NAME

pmb_configure.pl - PlayMOBY Install/Configuration script

=head1 SYNOPSIS

pmb_configure.pl --unix_path=<where you untar playmoby> --http_path=<same path but accessible via apache> --auth_uri=my.signature.uri [--timeout=<i minutes>] directory> --all|register|deregister|test [--central]

=head1 DESCRIPTION

When you install playmoby environnement for the first time you have to configure it.
The script pmb_configure.pl will modify many files (config files, perl modules) to tune playmoby according to your system.


% tcsh
% ./pmb_configure.pl --unix_path=/full/path/to/playmoby --http_path=http://your.web.server/path/to/playmoby --auth_uri=your.uri.signature [--timeout=i_minutes]


        --unix_path:   the full path where you untar playmoby tarball; in fact it's the directory where is "pmb_configure.pl" script

        --http_path:   this is the "apache path" to the playmoby directory; because of webservice technology , the rest of the world should access
                      to this directory using http protocol, and especially "rdf" directory and "cgi/dispatcher.cgi" should be accessible
                      NB1: do not forget to add ExecCGI option onto "cgi" directory
                      NB2: try to access these directories using a web browser (using http_path root URL) from an external IP class (to avoid problems of security access)

		           RDF directory: http://lipm-bioinfo.toulouse.inra.fr/path/to/playmoby/rdf/
		           CGI directory: http://lipm-bioinfo.toulouse.inra.fr/path/to/playmoby/cgi/index.cgi
		            

        --auth_uri:    the authority URI is a signature for your Webservices deployed under one instance of playmoby

        --email   :    this PlayMoby instance administrator email

        --timeout :    the timeout parameter corresponds to the SOAP request Time out (in minutes) (LWP::UserAgent timeout) ; by default, we use 10 minutes
                      there's also another timeout in your apache server, so take care to be synchronized with this one
                      if you want to modify apache timeout, contact your administrator (maybe he should prefer to use a virtual host with specific timeout)


The script pmb_configure.pl also gernerates a setenv (.setenv) file containing two environment variables: PMBBIN and PMBCFG

    * $PMBBIN points to the usefull playmoby scripts directory
    * $PMBCFG points to configuration directory (which contains especially central/registry configuration files)

To use these environment variables, type the following command each time you connect to your playmoby environment


% tcsh
% source .setenv


=head1 AUTHORS

 Sebastien.Carrere@toulouse.inra.fr 
 Sebastien.Letort@toulouse.inra.fr

=head1 SUBROUTINES

=cut


use strict;
my ($dirprg, $nameprg);

BEGIN
{

    ($dirprg, $nameprg) = $0 =~ /(.+)\/(.+)/;
    
    $dirprg = '.' if (! (defined $dirprg) || ($dirprg eq ''));
    
    if (-d "$dirprg/lib")
    {
        unshift(@INC, "$dirprg/lib");
    }
    if (-d "$dirprg/../lib")
    {
        unshift(@INC, "$dirprg/../lib");
    }
    if (-d "$dirprg/lib/Appli")
    {
        unshift(@INC, "$dirprg/lib/Appli");
    }
    

}

use Getopt::Long;
use ParamParser;
use BiomobyUtils;
use Cwd 'abs_path';
use HTML::Entities;
use IO::File;
use File::Which qw(which);
use Term::ANSIColor;


my $O_PARAM;
my @A_MANDATORY_BINARIES = ('xsltproc', 'mv', 'rm', 'find', 'Mail', 'cat', 'tar', 'bzip2', 'python');
$dirprg = abs_path($dirprg);
my $DEFAULT_REGISTRY = 'mobycentral';


MAIN:
{

    $O_PARAM = New ParamParser('GETOPTLONG', ("unix_path=s", "http_path=s", "auth_uri=s", "email=s", "timeout=i", "registry=s","verbose"));

    if (-e "$dirprg/.configure")
    {
	$O_PARAM->Update("$dirprg/.configure", 'A'); 
	print STDERR color 'bold';
    	print STDERR "Use already defined parameters from $dirprg/.configure\n";
    	print STDERR color 'reset';
    }

    $O_PARAM->SetUnlessDefined('unix_path',abs_path($dirprg));
    $O_PARAM->SetUnlessDefined('registry',$DEFAULT_REGISTRY);

    &ControlUsage('http_path');
    &ControlUsage('auth_uri');
    &ControlUsage('email');

    $O_PARAM->SetUnlessDefined('timeout', 10);

    my $unix_path_tmp = $O_PARAM->Get('unix_path');
    my $http_path     = $O_PARAM->Get('http_path');
    my $unix_path     = abs_path($unix_path_tmp);

    print STDERR "\nunix_path $unix_path\n";
    print STDERR "http_path $http_path\n\n";

    my $dispatcher_file_mask = $unix_path . '/cgi/dispatcher.cgi.mask';
    die("ERROR: $dispatcher_file_mask not found") if (!-e $dispatcher_file_mask);
    my $dispatcher_file = $unix_path . '/cgi/dispatcher.cgi';

    print STDERR "Substitute $dispatcher_file_mask\n";
    &Substitute($dispatcher_file_mask, $dispatcher_file);
    chmod 0755, $dispatcher_file;

    my $dispatcher_async_file_mask = $unix_path . '/cgi/dispatcher-async.cgi.mask';
    die("ERROR: $dispatcher_async_file_mask not found") if (!-e $dispatcher_async_file_mask);
    my $dispatcher_async_file = $unix_path . '/cgi/dispatcher-async.cgi';

    print STDERR "Substitute $dispatcher_async_file_mask\n";
    &Substitute($dispatcher_async_file_mask, $dispatcher_async_file);
    chmod 0755, $dispatcher_async_file;

    my $pmb_checkws_file_mask = $unix_path . '/bin/pmb_CheckWS.pl.mask';
    die("ERROR: $pmb_checkws_file_mask not found") if (!-e $pmb_checkws_file_mask);
    my $pmb_checkws_file = $unix_path . '/bin/pmb_CheckWS.pl';

    print STDERR "Substitute $pmb_checkws_file_mask\n";
    &Substitute($pmb_checkws_file_mask, $pmb_checkws_file);
    chmod 0755, $pmb_checkws_file;

    my $pmb_deploy_file_mask = $unix_path . '/bin/pmb_DeployWS.pl.mask';
    die("ERROR: $pmb_deploy_file_mask not found") if (!-e $pmb_deploy_file_mask);
    my $pmb_deploy_file = $unix_path . '/bin/pmb_DeployWS.pl';

    print STDERR "Substitute $pmb_deploy_file_mask\n";
    &Substitute($pmb_deploy_file_mask, $pmb_deploy_file);
    chmod 0755, $pmb_deploy_file;


    chmod 0777, $unix_path . '/logs/access_log';

    
    my $mobyleparser_file_mask = $unix_path . '/lib/MobyleParser.pm.mask';
    die("ERROR: $mobyleparser_file_mask not found") if (!-e $mobyleparser_file_mask);
    my $mobyleparser_file = $unix_path . '/lib/MobyleParser.pm';

    print STDERR "Substitute $mobyleparser_file_mask\n";
    &Substitute($mobyleparser_file_mask, $mobyleparser_file);

    my $pmbtest_file_mask = $unix_path . '/lib/PmbTest.pm.mask';
    die("ERROR: $pmbtest_file_mask not found") if (!-e $pmbtest_file_mask);
    my $pmbtest_file = $unix_path . '/lib/PmbTest.pm';

    print STDERR "Substitute $pmbtest_file_mask\n";
    &Substitute($pmbtest_file_mask, $pmbtest_file);

    my $LWP_UserAgent_file_mask = $unix_path . '/lib/LWP/UserAgent.pm.mask';
    die("ERROR: $LWP_UserAgent_file_mask not found") if (!-e $LWP_UserAgent_file_mask);
    my $LWP_UserAgent_file = $unix_path . '/lib/LWP/UserAgent.pm';

    print STDERR "Substitute $LWP_UserAgent_file_mask\n";
    &Substitute($LWP_UserAgent_file_mask, $LWP_UserAgent_file);

    my $index_file_mask = $unix_path . '/index.html.mask';
    die("ERROR: $index_file_mask not found") if (!-e $index_file_mask);
    my $index_file = $unix_path . '/index.html';

    print STDERR "Substitute $index_file_mask\n";
    &Substitute($index_file_mask, $index_file);
    
    my $checkws_file_mask = $unix_path . '/cfg/CheckWS.cfg.mask';
    die("ERROR: $checkws_file_mask not found") if (!-e $checkws_file_mask);
    my $checkws_file = $unix_path . '/cfg/CheckWS.cfg';

    print STDERR "Substitute $checkws_file_mask\n";
    &Substitute($checkws_file_mask, $checkws_file);

    my $reportxsl_file_mask = $unix_path . '/web/xsl/report.xsl.mask';
    die("ERROR: $reportxsl_file_mask not found") if (!-e $reportxsl_file_mask);
    my $reportxsl_file = $unix_path . '/web/xsl/report.xsl';

    print STDERR "Substitute $reportxsl_file_mask\n";
    &Substitute($reportxsl_file_mask, $reportxsl_file);
    
    my $dictionnary_file_mask = $unix_path . '/lib/Appli/data/BioMobyMobyleDictionnary.xml.mask';
    die("ERROR: $dictionnary_file_mask not found") if (!-e $dictionnary_file_mask);
    my $dictionnary_file = $unix_path . '/lib/Appli/data/BioMobyMobyleDictionnary.xml';

    print STDERR "Substitute $dictionnary_file_mask\n";
    &Substitute($dictionnary_file_mask, $dictionnary_file);
    
    
    my $javascript_file_mask = $unix_path . '/web/js/Tools.js.mask';
    die("ERROR: $javascript_file_mask not found") if (!-e $javascript_file_mask);
    my $javascript_file = $unix_path . '/web/js/Tools.js';

    print STDERR "Substitute $javascript_file_mask\n";
    &Substitute($javascript_file_mask, $javascript_file);

    my $asynctemplate_file_mask = $unix_path . '/tmpl/service-async.template.mask';
    die("ERROR: $asynctemplate_file_mask not found") if (!-e $asynctemplate_file_mask);
    my $asynctemplate_file = $unix_path . '/tmpl/service-async.template';

    print STDERR "Substitute $asynctemplate_file_mask\n";
    &Substitute($asynctemplate_file_mask, $asynctemplate_file);
    
    my $rh_mask_binaries = &CheckConfig();
    
    my $unix_file_mask = $unix_path . '/cfg/unix_cmd.cfg.mask';
    die("ERROR: $unix_file_mask not found") if (!-e $unix_file_mask);
    my $unix_file = $unix_path . '/cfg/unix_cmd.cfg';
    print STDERR "Substitute $unix_file_mask\n";
    &Substitute($unix_file_mask, $unix_file,$rh_mask_binaries);
   

    print STDERR "print .setenv file\n";
    my $fh_setenv = new IO::File(">$dirprg/.setenv");
    print $fh_setenv <<END;
setenv PMBCFG $unix_path/cfg
setenv PMBBIN $unix_path/bin
END
    $fh_setenv->close;

    $O_PARAM->Dump("$dirprg/.configure");



   
    print STDERR color 'bold green';

    print STDERR "\n\nTry to connect to "
          . $O_PARAM->Get('http_path'). "/index.html using your web browser to check configuration.\n\n\n";
          
        print STDERR "NB: You should have activate  CGI script execution on $unix_path/cgi directory before ... \n";
        print STDERR
          "The easiest way is to add Alias and  ScriptAlias directives in your apache configuration file like this one:\n\n";

        my $http_root = $O_PARAM->Get('http_path');
        $http_root =~ s/\/+$//;
        $http_root =~ s/http:\/\/[^\/]+//;
        print STDERR "\t\tScriptAlias \"" . $http_root . "/cgi/\" $unix_path/cgi/\n";
        print STDERR "\t\tAlias \"" . $http_root . "/\" $unix_path/\n\n";
        print STDERR color 'reset';


}




=head2 procedure Substitute

 Title        : Substitute

=cut

sub Substitute
{
    my ($in, $out, $rh_mask_binaries) = (@_);

    my $in_fh  = new IO::File($in);
    my $out_fh = new IO::File(">$out");
    die("$!: $in")  if (!defined $in_fh);
    die("$!: $out") if (!defined $out_fh);

    my $unix_path_tmp = $O_PARAM->Get('unix_path');
    my $unix_path     = abs_path($unix_path_tmp);
    my $http_path     = $O_PARAM->Get('http_path');
    my $auth_uri      = $O_PARAM->Get('auth_uri');
    my $timeout       = $O_PARAM->Get('timeout');
    my $email       = $O_PARAM->Get('email');
    my $registry       = $O_PARAM->Get('registry');


    $unix_path =~ s/\/$//g;
    $http_path =~ s/\/$//g;

    my ($http_root) =  ($http_path =~ /http:\/\/[^\/]+(\S+)/);


    while (my $line = <$in_fh>)
    {
        chomp $line;
        $line =~ s/#unix_path#/$unix_path/g;
        $line =~ s/#http_path#/$http_path/g;
        $line =~ s/#http_root#/$http_root/g;
        $line =~ s/#auth_uri#/$auth_uri/g;
        $line =~ s/#mask_timeout#/$timeout/g;
        $line =~ s/#mask_email#/$email/g;
        $line =~ s/#mask_registry#/$registry/g;

        if (defined $rh_mask_binaries)
	{
		foreach my $mask_binary (keys %{$rh_mask_binaries})
    		{
			$line =~ s/$mask_binary/$rh_mask_binaries->{$mask_binary}/g;
		}
	}
        
        print $out_fh $line . "\n";
    }

    $in_fh->close;
    $out_fh->close;

}

=head2 procedure CheckConfig

 Title        : CheckConfig

=cut

sub CheckConfig
{

    my @a_mandatory_modules = (
                               'UNIVERSAL',                 'Carp', 'File::Path',
                               'CGI',                       'Cwd',
                               'Data::Dumper',              'DBD::mysql',
                               'DBI',                       'Error',
                               'Error::IOException',        'Exporter',
                               'IO::Dir',                   'IO::File',
                               'File::Basename',            'File::Copy',
                               'File::Listing',             'File::Spec',
                               'File::Which',               'Getopt::Long',
                               'Getopt::Std',               'HTML::Entities',
                               'MIME::Base64',		    'String::Diff',
                               'XML::LibXML','version',
                               'SOAP::Transport::HTTP',
                               'SOAP::Lite',
                               'URI',                       'URI::Escape', 'XML::RSS',
                                                     'General',
                               'LipmError',                 'LipmError::ParameterException',
                               'LipmError::IOException',    'LipmError::ObjectException',
                               'Appli',                     'Biomoby::Object',
                               'Biomoby::Article',          'Biomoby::PrimaryArticle',
                               'Biomoby::SecondaryArticle', 'ParamParser',
                               'BiomobyUtils',              'MobyleElement', 'PmbTest'
                               );

    my %h_optional_modules = ('WSRF::Lite' => 'WSRF::Lite is mandatory to deploy asynchronous BioMoby web-services');

    my $config_ok = 1;

    my %h_mask_binaries = ();

    foreach my $mandatory_binary (@A_MANDATORY_BINARIES)
    {
        my $exe_path = which($mandatory_binary);

	$h_mask_binaries{'#mask_' . $mandatory_binary . '#'} = $exe_path;
	
        if (!defined($exe_path) || ($exe_path eq ''))
        {
            print STDERR color 'red';
            print STDERR "\nERROR CONFIGURATION: $mandatory_binary binary can not be found on your system\n\n";
            print STDERR color 'reset';
	    $config_ok = 0;
        }
        else
        {
            print STDERR color 'bold green';
            print STDERR "[OK] $mandatory_binary = $exe_path\n" if ($O_PARAM->IsDefined('verbose'));
            print STDERR color 'reset';

        }
    }

    foreach my $mandatory_module (@a_mandatory_modules)
    {
        eval "use $mandatory_module;";
        if ($@)
        {
            print STDERR color 'red';
            print STDERR "\nERROR CONFIGURATION: $mandatory_module can not be found on your system: $@\nYou can find it at CPAN URL: http://search.cpan.org/search?mode=module&query=$mandatory_module\n\n";
	    print STDERR color 'reset';
            $config_ok = 0;
        }
        else
        {
            print STDERR color 'bold green';
            print STDERR "[OK] $mandatory_module\n" if ($O_PARAM->IsDefined('verbose'));
            print STDERR color 'reset'; 
        }
    }
    
    foreach my $optional_module (keys %h_optional_modules)
    {
        eval "use $optional_module;";
        if ($@)
        {
            print STDERR color 'yellow';
            print STDERR "\nWARNING CONFIGURATION: $optional_module can not be found on your system: $@\n$h_optional_modules{$optional_module}\nYou can find it at CPAN URL: http://search.cpan.org/search?mode=module&query=$optional_module\n\n";
	    print STDERR color 'reset';
        }
        else
        {
            print STDERR color 'bold green';
            print STDERR "[OK] $optional_module\n" if ($O_PARAM->IsDefined('verbose'));
            print STDERR color 'reset'; 
        }
    }


    if ($config_ok == 0)
    {
	print STDERR color 'bold red';
    	print STDERR ("\n\nPRE-REQUISITE MODULES/BINARIES ARE MISSING. CONFIGURATION FAILED\n\n\n");
	print STDERR color 'reset';
	exit;
    }
    else
    {
	print STDERR color 'bold green';
    	print STDERR "\n\nCONFIGURATION IS OK\n\n\n";
	print STDERR color 'reset';
    }

    return \%h_mask_binaries;

}

=head2 procedure ControlUsage

 Title        : ControlUsage
 Usage        : &ControlUsage()
 Prerequisite : none
 Function     : Control usage of program
 Returns      : undef if OK
 Args         : $param_name Param to control
 Globals      : $O_PARAM

=cut

sub ControlUsage
{
    my $param_name = shift();

    
    if (!(defined($O_PARAM->Get($param_name))) || ($O_PARAM->Get($param_name) eq ''))
    {
        &PrintUsage();
        &Exit("ERROR: $param_name is mandatory");
    }
    elsif (($param_name eq 'http_path') && ($O_PARAM->Get($param_name) !~ /^http:\/\//))
    {
        &PrintUsage();
        &Exit("ERROR: http_path parameter value MUST start with 'http://'");
    }
    return;
}

=head2 procedure PrintUsage

 Title        : PrintUsage
 Usage        : &PrintUsage()
 Prerequisite : none
 Function     : print usage of the program
 Returns      : none
 Args         : none
 Globals      : none

=cut

sub PrintUsage
{
    my %h_registries = %{&GetAvailableRegistries()};

    my $registries = "\n\t\t\t";
    foreach my $registry_name (keys %h_registries)
    {
        $registries .= "*$registry_name = $h_registries{$registry_name}\n\t\t\t";
    }

    print STDERR <<END;


Usage:

$0 --http_path=\"<YOUR PlayMOBY URL>\" --auth_uri=\"<my.signature.uri>\"  --email=\"<my.email\@here.com>\" [--timeout=<i minutes>] [--unix_path="<where you untar playmoby tarball>\"] [--verbose]

        --http_path: YOUR PlayMOBY URL; same path but accessible via apache (try it with a browser)
	--auth_uri : your signature URI
	--email    : this PlayMoby instance administrator email

	--registry : default BioMOBY registry [$DEFAULT_REGISTRY]			
			available values : $registries

	--timeout  : timeout SOAP request (default: 10 minutes) 

        --unix_path: where you untar playmoby tarball [default: $dirprg]

	--verbose  : print more messages


END
}

=head2 fonction GetAvailableRegistries

 Title        : GetAvailableRegistries
 
=cut

sub GetAvailableRegistries
{
    opendir(CENTRAL, "$dirprg/cfg/central") || die("$! $dirprg/cfg/central");
    my @a_cfg = grep {/\w+\.cfg$/} readdir(CENTRAL);
    closedir CENTRAL;
    my %h_central;

    foreach my $cfg_file (@a_cfg)
    {
        my $o_pp = New ParamParser("$dirprg/cfg/central/$cfg_file");
        $cfg_file =~ s/\.cfg$//;
        $h_central{$cfg_file} = '';

        if ($o_pp->IsDefined('mobycentral_description'))
        {
            $h_central{$cfg_file} = $o_pp->Get('mobycentral_description');
        }
    }

    return \%h_central;

}
