#! /usr/bin/perl
#
# $Id$
#

=pod

=head1 NAME

Authpasswd.pl - Create or update a passwd file suitable for the Auth module 

=head1 SYNOPSIS

Authpasswd.pl [--create] [--password passwd] [--priv priv_number] [--mail mail_address] [--workspace home_directory] passwordfile username

=head1 DESCRIPTION

 Authpasswd.pl opens (or create if the --create switch is specifed) a password file suitable for Auth
 Specify --passwd, --priv, --mail --workspace to set/modify the corresponding field
 A worspace is some sort of home directory
 A privilege is a number (integer, >=0) the precise meaning of this number is set by the client application
 Only the login field cannot be changed: if new, a line is created. If not new, the corresponding fiels are changed
 
=cut

package main;
use strict;
use warnings;
use IO::File;
use Digest::MD5 qw(md5 md5_hex md5_base64);

BEGIN
{
    if (exists $ENV{'LIPMLIB'})
    {
        push(@INC, $ENV{'LIPMLIB'});
    }
    else
    {
        my ($dirprg, $nameprg) = $0 =~ /(.+)\/(.+)/;
        push(@INC, "$dirprg/../lib", "$dirprg/lib");
    }
    our $VERSION = do {my @r = (q$Rev: 57 $ =~ /\d+/g); $r[0]};
}

use ParamParser;

MAIN:
{
    &Usage() if (@ARGV == 0);

    ParamParser::SetDefaultBehaviour('exit_on_getopt_error');
    ParamParser::SetDefaultUsage(\&Usage);

    my $o_param =
      New ParamParser('GETOPTLONG', "create", "password:s", "priv:i", "ip=s", "mail:s", "workspace:s", "login=s", "file=s");
    &Usage() if (!$o_param->IsDefined('login') or !$o_param->IsDefined('file'));

    # Open (may be create) the password file... or die
    my ($fh_passwd_in, $fh_passwd_out) = &OpenPasswdFile($o_param);

    # Write a new password file
    &UpdatePasswdFile($fh_passwd_in, $fh_passwd_out, $o_param);

    # Close and rename files
    $fh_passwd_out->close();
    if (defined($fh_passwd_in))
    {
        $fh_passwd_in->close();
        unlink($o_param->Get('file'));
        link($o_param->Get('file') . '.new', $o_param->Get('file'));
        unlink($o_param->Get('file') . '.new');
    }
}

=head2 function OpenPasswdFile

 Title		  : OpenPasswdFile
 Usage		  : my $fh_passwd = &OpenPasswdFile($o_param);
 Prerequisite : $o_param must be initialized
 Function	  : If --create specified die if the passwd file exists, else try to create it
                If --create NOT specified die if passwd file does not exist, else try top open it 
 Returns	  : ($fh_file_in, $fh_file_out) => 2 file handlers: the first open readonly, may be undef, the 2nd opend write only 
 Error        : dies if there is any problem

=cut

sub OpenPasswdFile
{
    my ($o_param) = @_;

    my ($fh_file_in, $fh_file_out);
    my $file = $o_param->Get('file');
    if ($o_param->IsDefined('create'))
    {
        die("ERROR - $file already exists") if (-e $file);
        $fh_file_in = undef;
        $fh_file_out = new IO::File(">>$file") or die("ERROR - Cannot open $file for writing");
    }
    else
    {
        die("ERROR - $file does not exist") unless (-e $file);
        $fh_file_in = new IO::File($file) or die("ERROR - Cannot open $file for reading");
        $file .= ".new";
        $fh_file_out = new IO::File(">$file") or die("ERROR - Cannot open $file for writing");
    }
    return ($fh_file_in, $fh_file_out);
}

=head2 procedure UpdatePasswdFile

 Title		  : UpdatePasswdFile
 Usage		  : &OpenPasswdFile($fh_passwd_in, $fh_passwd_out,$o_param);
 Prerequisite : $o_param must be initialized, $fh_passwd_in and out must be opened
 Procedure 	  : Copy _in to _out, modifying the line of user
 Args         : $fh_passwd_in  The old file, opened for reading (may be undef)
                $fh_passwd_out the new file, opened for writing
                $o_param

=cut

sub UpdatePasswdFile
{
    my ($fh_passwd_in, $fh_passwd_out, $o_param) = @_;

    my $login = $o_param->Get('login');

    my $modified = 0;
    if (defined($fh_passwd_in))
    {
        while (my $line = <$fh_passwd_in>)
        {

            # not this user: just copy the line
            if ($line !~ /^${login}:/)
            {
                print $fh_passwd_out $line;
            }
            else
            {
                chomp $line;
                my @a_f = split(':', $line);
                my $password  = $o_param->IsDefined('password')  ? md5_hex($o_param->Get('password')) : $a_f[1];
                my $ip      = $o_param->IsDefined('ip')      ? $o_param->Get('ip')              : $a_f[2];
                my $priv      = $o_param->IsDefined('priv')      ? $o_param->Get('priv')              : $a_f[3];
		my $mail      = $o_param->IsDefined('mail')      ? $o_param->Get('mail')              : $a_f[4];
                my $workspace = $o_param->IsDefined('workspace') ? $o_param->Get('workspace')         : $a_f[5];
                print $fh_passwd_out $login . ':'
                  . $password . ':'
                  . $ip . ':'
                  . $priv . ':'
                  . $mail . ':'
                  . $workspace
                  . ':/bin/tcsh' . "\n";
                $modified = 1;
            }
        }
    }

    # The line is not yet written: may be it is a new user, or it is a new file
    if ($modified == 0)
    {
        my $login     = $o_param->Get('login');
        my $password  = $o_param->IsDefined('password') ? md5_hex($o_param->Get('password')) : '*';
        my $ip      = $o_param->IsDefined('ip') ? $o_param->Get('ip') : 1575;
        my $priv      = $o_param->IsDefined('priv') ? $o_param->Get('priv') : 0;
        my $mail      = $o_param->IsDefined('mail') ? $o_param->Get('mail') : '';
        my $workspace = $o_param->IsDefined('workspace') ? $o_param->Get('workspace') : '';
        print $fh_passwd_out $login . ':'
          . $password . ':'
          . $ip . ':'
          . $priv . ':'
          . $mail . ':'
          . $workspace
          . ':/bin/tcsh' . "\n";
    }
}

sub Usage
{
    print
      "Usage: Authpasswd.pl [--create] [--password passwd] [--priv priv_number] [--mail mail_address] [--workspace home_directory] --file passwordfile --login username\n";
    print "           --create    Create the file (an error message is sent if the file already exists)\n";
    print "           --password  set the password of the user\n";
    print "           --ip        set the ip adress of the user\n";
    print "           --priv      set the privilege level (a positive integer) of the user\n";
    print "           --mail      set the address of the user (must be toto\@sometihng.org)\n";
    print "           --workspace set the workspace (hoem directory) of the user\n";
    exit 1;
}

